unit main;

// Advanced Sample for ShellBrowser
// This sample project looks like the windows explorer. The FolderIdList
// of each folder is stored in the Data property of each TreeNode

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Buttons, ShellBrowser, Menus, ShellApi;

type
  TMainForm = class(TForm)
    ListView: TListView;
    Panel1: TPanel;
    SmallImages: TImageList;
    TreeView: TTreeView;
    Splitter: TPanel;
    StatusBar1: TStatusBar;
    LargeImages: TImageList;
    PathLabel: TLabel;
    PropertiesButton: TSpeedButton;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    ListViewPopup: TPopupMenu;
    Rename1: TMenuItem;
    TreeViewPopup: TPopupMenu;
    FullExpand1: TMenuItem;
    FullCollapse1: TMenuItem;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    DeleteButton: TSpeedButton;
    ParentButton: TSpeedButton;
    ShellBrowser: TShellBrowser;
    SpeedButton5: TSpeedButton;
    procedure ListViewDblClick(Sender: TObject);
    procedure PropertiesButtonClick(Sender: TObject);
    procedure ListViewMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SplitterMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SplitterMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure SplitterMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure TreeViewDeletion(Sender: TObject; Node: TTreeNode);
    procedure TreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure SpeedButton1Click(Sender: TObject);
    procedure TreeViewExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure TreeViewMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ListViewEdited(Sender: TObject; Item: TListItem;
      var S: String);
    procedure Rename1Click(Sender: TObject);
    procedure FullExpand1Click(Sender: TObject);
    procedure FullCollapse1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure ParentButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure SpeedButton5Click(Sender: TObject);
    procedure TreeViewCompare(Sender: TObject; Node1, Node2: TTreeNode;
      Data: Integer; var Compare: Integer);
  private
    SplitterMove : Boolean;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation
{$R *.DFM}

procedure TMainForm.FormShow(Sender: TObject);
var drives, node, C_drive: TTreeNode;
begin
  // Simple implemetation of a splitter
  SplitterMove := False;
  // Make 'My Computer' active folder. method 1
  ShellBrowser.Folder := '';
  // Browse to Desktop
  ShellBrowser.SelectParent;
  // Make node for 'My Computer'
  drives := TreeView.Items.Add(nil, ShellBrowser.GetShellObjectName);
  drives.ImageIndex := ShellBrowser.IconNumber;
  drives.SelectedIndex := ShellBrowser.SelectedIconNumber;
  drives.Data := ShellBrowser.FolderIdList;
  C_Drive := drives;

  // Make 'My Computer' active folder again, method 2
  ShellBrowser.BrowseSpecialFolder(SF_DRIVES);
  // Enumerate Drives
  While ShellBrowser.Next do begin
    if not ShellBrowser.IsFolder then continue;
    node := TreeView.Items.AddChild(drives, ShellBrowser.GetShellObjectName);
    node.ImageIndex := ShellBrowser.IconNumber;
    node.SelectedIndex := ShellBrowser.SelectedIconNumber;
    node.HasChildren := ShellBrowser.HasSubFolders;
    node.Data := ConcatItemIdLists(ShellBrowser.FolderIdList,ShellBrowser.ItemIdList);
    if Pos('C:', node.Text)>0 then C_drive := Node;  // Remember Drive C:
  end;//while
  drives.Expand(False);
  drives.Alphasort;
  C_drive.Selected := True;
end;

// opens object after double click
procedure TMainForm.ListViewDblClick(Sender: TObject);
var i: Integer;
begin
  if not Assigned(ListView.Selected) then exit;
  if ShellBrowser.IsFolder then begin // if Folder then show it
    TreeView.Selected.Expand(False);
    for i:=0 to TreeView.Selected.Count-1 do  // search corresponding Tree Node
      if TreeView.Selected.Item[i].Text = ListView.Selected.Caption then begin
        TreeView.Selected.Item[i].Selected := True;
        break;
      end;
  end else begin
    ShellBrowser.ObjectName := ListView.Selected.Caption;
    if not ShellBrowser.InvokeContextMenuCommand('Default') then
      ShowMessage('Error! Could not execute default action.');
  end;
end;

// OnClick of the Properties button
procedure TMainForm.PropertiesButtonClick(Sender: TObject);
var i: Integer;
begin
  if ListView.SelCount=0 then exit;
  for i:=ListView.Selected.Index to ListView.Items.Count-1 do
    if ListView.Items[i].Selected then
      ShellBrowser.MultiObjects.Add(ListView.Items[i].Caption);
  if not ShellBrowser.InvokeContextMenuCommand('Properties') then
    ShowMessage('No Properties Available');
end;

// Use onMouseUp for displaying the context menu
procedure TMainForm.ListViewMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var Item   : TListItem;
    command: String;
    i      : Integer;
begin
  if Button <> mbRight then exit;
  Item := ListView.GetItemAt(X,Y);
  if not Assigned(Item) then exit; // No Item hit
  for i:=ListView.Selected.Index to ListView.Items.Count-1 do
    if ListView.Items[i].Selected then
      ShellBrowser.MultiObjects.Add(ListView.Items[i].Caption);
  ShellBrowser.SetFolderIdList(TreeView.Selected.Data);
  // This is a workaround for Delphi 2, because Pointer properties do not work
  // correctly here. Delphi 3 users should write:
  // ShellBrowser.FolderIdList := TreeView.Selected.Data;

  command := ShellBrowser.ShowContextMenu(ListView.ClientToScreen(Point(x,y)), ListViewPopup);
  if CompareText(command, 'delete')=0 then
    while Assigned(ListView.Selected) do
      ListView.Selected.Delete;
end;

procedure TMainForm.SplitterMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  SplitterMove := True;
end;

procedure TMainForm.SplitterMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if SplitterMove then
    TreeView.Width := ScreenToClient(Splitter.ClientToScreen(Point(X,Y))).X;
end;

procedure TMainForm.SplitterMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  SplitterMove := False;
end;

procedure TMainForm.TreeViewDeletion(Sender: TObject; Node: TTreeNode);
begin
  Allocator.Free(Node.Data);
end;

procedure TMainForm.TreeViewChange(Sender: TObject; Node: TTreeNode);
begin
  ShellBrowser.SetFolderIdList(Node.Data);
  // This is a workaround for Delphi 2, because Pointer properties do not work
  // correctly here. Delphi 3 users should write:
  // ShellBrowser.FolderIdList := TreeView.Selected.Data;

  PathLabel.Caption := ShellBrowser.Folder;
  ShellBrowser.FillListView(ListView, True);
end;

procedure TMainForm.SpeedButton1Click(Sender: TObject);
begin
  ListView.ViewStyle := TViewStyle(Ord(vsIcon) + TComponent(Sender).Tag);
end;

procedure TMainForm.TreeViewExpanding(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
var NewNode: TTreeNode;
begin
  if node.Count>0 then exit; // Already Filled?
  ShellBrowser.SetFolderIdList(Node.Data);
  // This is a workaround for Delphi 2, because Pointer properties do not work
  // correctly here. Delphi 3 users should write:
  // ShellBrowser.FolderIdList := TreeView.Selected.Data;

  While ShellBrowser.Next do begin
    if not ShellBrowser.IsFolder then continue;
    NewNode := TreeView.Items.AddChild(Node, ShellBrowser.GetShellObjectName);
    NewNode.ImageIndex := ShellBrowser.IconNumber;
    NewNode.SelectedIndex := ShellBrowser.SelectedIconNumber;
    NewNode.HasChildren := ShellBrowser.HasSubFolders;
    NewNode.Data := ConcatItemIdLists(ShellBrowser.FolderIdList,ShellBrowser.ItemIDList);
  end;//while
  Node.Alphasort;
end;

procedure TMainForm.TreeViewMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var Node: TTreeNode;
begin
  if Button <> mbRight then exit;
  Node := TreeView.GetNodeAt(X,Y);
  if not Assigned(Node) then exit; // No Item hit
{ifndef VER100}
  Node.Selected := True;
{endif}
  ShellBrowser.SetFolderIdList(Node.Data);
  // This is a workaround for Delphi 2, because Pointer properties do not work
  // correctly here. Delphi 3 users should write:
  // ShellBrowser.FolderIdList := TreeView.Selected.Data;

  ShellBrowser.SelectParent;
  if ShellBrowser.ShowContextMenu(TreeView.ClientToScreen(Point(x,y)), TreeViewPopup) = 'delete' then
    Node.Delete;
end;

procedure TMainForm.ListViewEdited(Sender: TObject; Item: TListItem; var S: String);
begin
  ShellBrowser.SetFolderIdList(TreeView.Selected.Data);
  // This is a workaround for Delphi 2, because Pointer properties do not work
  // correctly here. Delphi 3 users should write:
  // ShellBrowser.FolderIdList := TreeView.Selected.Data;

  ShellBrowser.ObjectName := Item.Caption;
  ShellBrowser.RenameObject(S);
end;

procedure TMainForm.Rename1Click(Sender: TObject);
begin
  ListView.Selected.EditCaption;
end;

procedure TMainForm.FullExpand1Click(Sender: TObject);
begin
  TreeView.Selected.Expand(True);
end;

procedure TMainForm.FullCollapse1Click(Sender: TObject);
begin
  TreeView.Selected.Collapse(True);
end;

procedure TMainForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.About1Click(Sender: TObject);
begin
  ShellAbout(Handle, PChar('Advanced Sample for TShellBrowser#'), PChar('Advanced sample for TShellBrowser'#10'More info at: http://www.jam-software.com'), Application.Icon.Handle);
end;

procedure TMainForm.ParentButtonClick(Sender: TObject);
begin
  if Not Assigned(TreeView.Selected) or not Assigned(TreeView.Selected.Parent) then exit;
  TreeView.Selected.Parent.Selected := True;
end;

procedure TMainForm.DeleteButtonClick(Sender: TObject);
begin
  if not Assigned(ListView.Selected) then exit;
  ShellBrowser.ObjectName := ListView.Selected.Caption;
  if ShellBrowser.InvokeContextMenuCommand('Delete') then
    ListView.Selected.Delete;
end;

procedure TMainForm.SpeedButton5Click(Sender: TObject);
begin
  ShellBrowser.ShowNetConnectionDialog;
end;

procedure TMainForm.TreeViewCompare(Sender: TObject; Node1,
  Node2: TTreeNode; Data: Integer; var Compare: Integer);
var p1,p2: Integer;
begin
  p1 := Pos(':', Node1.Text);
  p2 := Pos(':', Node2.Text);
  if (p1>0) and (p2>0) then // Both are drives
    Compare := CompareText(Node1.Text[p1-1],Node2.Text[p2-1])
  else if p1>0 then
    Compare := -1
  else if p2>0 then
    Compare := 1
  else
    Compare := CompareText(Node1.Text, Node2.Text);
end;

end.

